@@ -1098,6 +1098,31 @@ class Analytics(object):
         except StopIteration as stop:
             return
 
+    def fetch_export_links(self, id):
+        resp_json = self.__config.auth_get(self.__config.get("MANTA_URL") + "aegean_batch_runs/" + str(id)).json()
+        provider_results = resp_json["batch_run_provider_versions"]
+        exports = [] # list of { provider_name, export_link }
+        for r in provider_results:
+            export_link = r["most_recent_export_link"]
+            provider_name = r["table_provider"]["name"]
+            if export_link:
+                exports.append({ "provider_name": provider_name,
+                                 "export_link": export_link })
+            else:
+                print("Provider " +provider_name+ " has not finished enrichment and will not be included." , file=sys.stderr)
+        doc = Doc()
+        with doc.tag("table"):
+            with doc.tag("tr"):
+                doc.line("th", "Export Link")
+            if len(exports) > 0:
+                for i, export in enumerate(exports):
+                    with doc.tag("tr"):
+                        with doc.tag("td"):
+                            doc.line("a", export["provider_name"], href=export["export_link"])
+            else:
+                doc.line("td", "No exports available")
+        return HTML(doc.getvalue())
+
     def search(self, inputs=None, hosted_input=None, tags=None, view="html", strict=False):
         """
         Looks for providers that are able to return data for the
