@@ -2,19 +2,11 @@ import pandas as pd
 import numpy as np
 import re
 import json
+from demyst.common import is_na, is_value
 
 # These pseudo-attributes are removed from the final report
 PSEUDO_ATTRS=["client_id", "error", "is_hit", "row_id"]
 
-# Treat Pandas/Numpy NaN, as well as strings containing only
-# whitespace, as not-a-value.  Everything else is considered a value.
-def is_na(value):
-    return pd.isna(value) or (isinstance(value, str) and value.strip() == "")
-
-# For convenience
-def is_value(value):
-    return not is_na(value)
-
 # For each product, maintain one product report.  It stores the number
 # of matching rows and error rows, and also one attribute report for
 # each attribute.
@@ -48,12 +40,15 @@ class AttrReport(object):
 def do_report(df):
 
     # Preprocess rows into a list of nested dictionaries for easier
-    # handling.  For each row, there is one list element.  It contains
-    # one dictionary for each product in that row.  It contains the
-    # attributes for that product from that row.  List attributes
-    # (attr[0], attr[1], ...)  are collapsed into a single entry
-    # (attr[*]), therefore the values must be a list.
+    # handling.  For each row, there is one list element, called
+    # product row.  It is a dictionary with a key for each product in
+    # that row.  It contains the attributes for that product from that
+    # row.  List attributes (attr[0], attr[1], ...)  are collapsed
+    # into a single entry (attr[*]), therefore the attribute values
+    # are stored as a list.
+    #                     |-------------- product row -----------------|
     product_rows = [] # [ { product_name -> { attr_name -> [ value ] } } ]
+    #                                       | attributes for product |
     for _, df_row in df.iterrows():
         product_row = {}
         for key, value in df_row.items():
@@ -71,7 +66,12 @@ def do_report(df):
 
     # Process individual product rows into product reports
     product_reports = {} # product_name -> ProductReport
+    # Counter of rows that match all products
+    matches_all = 0
     for product_row in product_rows:
+        # For calculation of matches_all:
+        num_products = len(product_row)
+        matches_products = 0 # How many products does this row match?
         for product_name, attrs in product_row.items():
             # Get or create product report
             if product_name in product_reports:
@@ -86,6 +86,7 @@ def do_report(df):
             # Row is a match for that product: increase match count
             elif is_match(attrs):
                 product_report.match_ct += 1
+                matches_products += 1
             # Process attributes.
             #
             # Note that we do this even if the row is an error.  The
@@ -116,6 +117,9 @@ def do_report(df):
                         # one value.
                         attr_report.fill_ct += 1
                         attr_report.values.extend(values)
+        # If the row matched all providers, increase total count.
+        if matches_products == num_products:
+            matches_all += 1
 
     # Final step: produce report DF from product reports
     out_rows = [] # list of rows of report DF
@@ -127,12 +131,14 @@ def do_report(df):
             # because dataframes implement all the utility methods
             # like std and variance that we need, and series don't.
             values = pd.DataFrame(attr_report.values, columns=["col"])[["col"]]
+            # Round to 2 places
+            rnd = lambda x: round(x, 2)
             out_row = {
                 "product_name": product_name,
-                "product_match_rate": float(product_report.match_ct) / float(total_ct) * 100.0,
-                "product_error_rate": float(product_report.error_ct) / float(total_ct) * 100.0,
+                "product_match_rate": rnd(float(product_report.match_ct) / float(total_ct) * 100.0),
+                "product_error_rate": rnd(float(product_report.error_ct) / float(total_ct) * 100.0),
                 "attribute_name": attr_name,
-                "attribute_fill_rate": float(attr_report.fill_ct) / float(total_ct) * 100.0,
+                "attribute_fill_rate": rnd(float(attr_report.fill_ct) / float(total_ct) * 100.0),
                 "std": do_standard_method(values, "std"),
                 "median": do_standard_method(values, "median"),
                 "mean": do_standard_method(values, "mean"),
@@ -142,11 +148,12 @@ def do_report(df):
                 "attribute_type": values["col"].dtypes,
                 "unique_values": values["col"].nunique(),
                 "most_common_values": values["col"].value_counts().head(5).to_dict(),
-                "cardinality": cardinality(values)
+                "cardinality": cardinality(values),
+                "matches_all": matches_all
                 }
             out_rows.append(out_row)
     # Put columns into proper order
-    return pd.DataFrame(out_rows)[['product_name', 'product_match_rate', 'product_error_rate', 'attribute_name', 'attribute_fill_rate', 'attribute_type', 'unique_values', 'most_common_values', 'cardinality', 'std', 'median', 'mean', 'max_value', 'min_value', 'variance']]
+    return pd.DataFrame(out_rows)[['product_name', 'product_match_rate', 'product_error_rate', 'attribute_name', 'attribute_fill_rate', 'attribute_type', 'unique_values', 'most_common_values', 'cardinality', 'std', 'median', 'mean', 'max_value', 'min_value', 'variance', 'matches_all']]
 
 # Check if a row is considered a match.
 #
@@ -156,9 +163,9 @@ def do_report(df):
 #
 # (* per is_value)
 def is_match(attrs):
-    # Stupid way of writing this due to attribute values being arrays.
-    if ("is_hit" in attrs) and any((is_value(val) and val) for val in attrs["is_hit"]):
-        return True
+    if ("is_hit" in attrs):
+        # Stupid way of writing this due to attribute values being arrays.
+        return any((is_value(val) and val) for val in attrs["is_hit"])
     else:
         for attr_name, attr_values in attrs.items():
             # Ignore pseudo-columns
